﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.SqlClient;
using System.Configuration;
using System.Net;
using System.Data;

namespace Nansen.StatusCodes.Persistence
{
    public class SimpleSqlUrlMapper: IUrlMapper
    {
        private static string _connectionStringName;
        static SimpleSqlUrlMapper()
        {
            _connectionStringName = EPiServer.Configuration.Settings.Instance.ConnectionStringName;
        }

        private const string TABLE_NAME = "UrlStatusCodeMappings";

        private SqlConnection GetConnection()
        {
            return new SqlConnection(ConfigurationManager.ConnectionStrings[_connectionStringName].ConnectionString);
        }

        public void GenerateSchema()
        {
            SqlConnection connection = this.GetConnection();
            string query = string.Format("if not exists (select * from sysobjects where name = '{0}' and xtype='U') create table {0} ( ID smallint identity(1,1) primary key clustered, OldUrl varchar(MAX), NewUrl varchar(MAX) )", TABLE_NAME);
            SqlCommand command = new SqlCommand(query, connection);

            try
            {
                connection.Open();
                command.ExecuteNonQuery();
            }
            finally
            {
                connection.Close();
            }
        }

        public void Add301MovedPermanentlyStatus(string oldUrl, string newUrl)
        {
            SqlConnection connection = this.GetConnection();

            string query = string.Format(
                @"update {0} set NewUrl = @NewUrl where NewUrl = @OldUrl collate SQL_Latin1_General_CP1_CI_AS;
                  insert into {0} (OldUrl, NewUrl) values (@OldUrl, @NewUrl);
                  delete from {0} where OldUrl = @NewUrl collate SQL_Latin1_General_CP1_CI_AS;", TABLE_NAME);

            SqlCommand command = new SqlCommand(query, connection);
            command.Parameters.Add("@OldUrl", System.Data.SqlDbType.VarChar, -1).Value = oldUrl;
            command.Parameters.Add("@NewUrl", System.Data.SqlDbType.VarChar, -1).Value = newUrl;

            try
            {
                connection.Open();
                command.ExecuteNonQuery();
            }
            finally
            {
                connection.Close();
            }
        }

        public void Add410GoneStatus(string url)
        {
            SqlConnection connection = this.GetConnection();

            string query = string.Format(
                @"if exists (select * from {0} where OldUrl = @Url or NewUrl = @Url collate SQL_Latin1_General_CP1_CI_AS) 
                     update {0} set NewUrl = @Null where OldUrl = @Url or NewUrl = @Url
                  else 
                     insert into {0} (OldUrl, NewUrl) values (@Url, @Null)", TABLE_NAME);

            SqlCommand command = new SqlCommand(query, connection);
            command.Parameters.Add("@Url", System.Data.SqlDbType.VarChar, -1).Value = url;
            command.Parameters.Add("@Null", System.Data.SqlDbType.VarChar, -1).Value = DBNull.Value;

            try
            {
                connection.Open();
                command.ExecuteNonQuery();
            }
            finally
            {
                connection.Close();
            }
        }

        public void RemoveMappings(string url)
        {
            SqlConnection connection = this.GetConnection();

            string query = string.Format("delete from {0} where OldUrl = @Url", TABLE_NAME);
            SqlCommand command = new SqlCommand(query, connection);
            command.Parameters.Add("@Url", System.Data.SqlDbType.VarChar, -1).Value = url;

            try
            {
                connection.Open();
                command.ExecuteNonQuery();
            }
            finally
            {
                connection.Close();
            }
        }

        public bool Exists(string url)
        {
            SqlConnection connection = this.GetConnection();

            string query = string.Format("select * from {0} where OldUrl = @Url", TABLE_NAME);
            SqlCommand command = new SqlCommand(query, connection);
            command.Parameters.Add("@Url", System.Data.SqlDbType.VarChar, -1).Value = url;

            bool exists;
            try
            {
                connection.Open();
                exists = command.ExecuteScalar() != null;
            }
            finally
            {
                connection.Close();
            }

            return exists;
        }

        public string GetRedirectUrl(string oldUrl)
        {
            SqlConnection connection = this.GetConnection();

            string query = string.Format("select * from {0} where OldUrl = @Url and NewUrl is not null", TABLE_NAME);
            SqlCommand command = new SqlCommand(query, connection);
            command.Parameters.Add("@Url", System.Data.SqlDbType.VarChar, -1).Value = oldUrl;

            connection.Open();
            SqlDataAdapter adapter = new SqlDataAdapter(command);
            DataSet dataSet = new DataSet();
            adapter.Fill(dataSet);

            return dataSet.Tables[0].Rows[0]["NewUrl"] as string;
        }

        public HttpStatusCode? GetStatusCode(string url)
        {
            if (Is410Gone(url))
            {
                return HttpStatusCode.Gone;
            }
            else if (Is301MovedPermanently(url))
            {
                return HttpStatusCode.MovedPermanently;
            }

            return null;
        }

        private bool Is410Gone(string url)
        {
            SqlConnection connection = this.GetConnection();

            string query = string.Format("select * from {0} where OldUrl = @Url and NewUrl is null", TABLE_NAME);
            SqlCommand command = new SqlCommand(query, connection);
            command.Parameters.Add("@Url", System.Data.SqlDbType.VarChar, -1).Value = url;

            bool exists;
            try
            {
                connection.Open();
                exists = command.ExecuteScalar() != null;
            }
            finally
            {
                connection.Close();
            }

            return exists;
        }

        private bool Is301MovedPermanently(string url)
        {
            SqlConnection connection = this.GetConnection();

            string query = string.Format("select * from {0} where OldUrl = @Url and NewUrl is not null", TABLE_NAME);
            SqlCommand command = new SqlCommand(query, connection);
            command.Parameters.Add("@Url", System.Data.SqlDbType.VarChar, -1).Value = url;
            command.Parameters.Add("@Null", System.Data.SqlDbType.VarChar, -1).Value = DBNull.Value;

            bool exists;
            try
            {
                connection.Open();
                exists = command.ExecuteScalar() != null;
            }
            finally
            {
                connection.Close();
            }

            return exists;
        }
    }
}
